<?php
  /**
   * Standard event handler for context cursor manipulation.
   *
   * @access public
   * @module record
   * @package User interface
   */

  # $Id: records.php,v 1.57 2002/06/01 17:13:43 sven Exp $
  #
  # SSI-independent built-in views for creating and deleting records.
  #
  # Copyright (c) 2000-2002 dev/consulting GmbH
  #		            Sven Michael Klose (sven@devcon.net)
  #
  # This library is free software; you can redistribute it and/or
  # modify it under the terms of the GNU Lesser General Public
  # License as published by the Free Software Foundation; either
  # version 2.1 of the License, or (at your option) any later version.
  #
  # This library is distributed in the hope that it will be useful,
  # but WITHOUT ANY WARRANTY; without even the implied warranty of
  # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  # Lesser General Public License for more details.
  #
  # You should have received a copy of the GNU Lesser General Public
  # License along with this library; if not, write to the Free Software
  # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  # About this file:
  #
  # This view functions do simple operations on records using the simple
  # data getters and setters of admin_panel.class. The links are
  # generated by widget functions in admin_panel.class.

  /**
   * Initialise this module.
   *
   * @param object application $this
   */
  function _records_init (&$this)
  {
    $this->add_function ('record_create');
    $this->raw_views['record_create'] = true;

    $this->add_function ('record_update');
    $this->raw_views['record_update'] = true;

    $this->add_function ('record_delete');
    $this->raw_views['record_delete'] = true;

    $this->add_function ('record_delete_force');
    $this->raw_views['record_delete_force'] = true;

    $this->record_messages = array (
      'record_saved' => 'Record saved.',
      'delete_ask' => 'Delete this record?',
      'delete_nothing' => 'Nothing delted.',
      'delete_done' => 'Record removed.',
      'create_done' => 'Record created.',
      'yes' => 'Yes.',
      'no' => 'No.'
    );
  }

  /**
   * Create record listed in $sources with optional field values or alieases.
   *
   * @access public
   * @param object application $this
   * @param array $set Source set which describes the records to be created.
   */
  function record_create_set (&$this, &$set)
  {
    $ui =& admin_panel::instance ();

    $keyset = array ();
    foreach ($set as $cursortype => $sources) {
      $cursor =& cursor::factory ($cursortype);

      foreach ($sources as $source => $pre) {
        if (!is_array ($pre))
          die ("Can't create record. Preset values are not in an array for " .
               "source $source.");

        # Replace aliases by key to inserted record in specified source in
        # this function. An alias is placed in preset_values' field data in
        # the form @<source name>.
        if (sizeof ($pre)) {
          foreach ($pre as $field => $data) {
            if (substr ($data, 0, 1) == '@') {
              $s = substr ($data, 1);
              if (!isset ($aliases[$s]))
                die ("No record in source $source created source $s could " .
                     "point to.");

              # Replace alias by id of inserted record.
              $pre[$field] = $aliases[$s];
            }
          }
        }

        # Create record.
        $cursor->set_source ($source);
        $key = $cursor->create ($pre);
        $keyset[$cursor->type ()][$source] = $key;

        # Remeber key for alias.
        $aliases[$source] = $key;

        # Highlight new record.
        $cursor->set_key ($key);
        $ui->highlight[$cursor->id ()] = '#00FF00';
      }
    }

    return $keyset;
  }

  /**
   * Create records, print messagebox and jump to next view.
   *
   * @param object application $this
   * @param array Keyset created by record_create_set().
   * @see record_create_set()
   */
  function _record_create_continue (&$this, $keys)
  {
    $next_view =& $this->arg ('next_view', ARG_OPTIONAL);
    $ret =& $this->arg ('ret', ARG_OPTIONAL);

    $ui =& admin_panel::instance ();

    if (!$keys)
      die ('Couldn\'t create record.');

    if ($keys && isset ($this->record_messages['create_done']))
      $ui->msgbox ($this->record_messages['create_done']);

    # Call next function.
    if ($next_view) {
      if (is_string ($next_view))
        $next_view =& new event ($next_view);
      if ($ret) {
        $k = $keys;
        $k = array_pop ($k);
        reset ($k);
        $next_view->args[$ret] = $k[key ($k)];
      }
      $this->call ($next_view);
    }

    return $keys;
  }

  /**
   * Event handler: Create a new row in table.
   *
   * Optional event argument 'sources' takes a source set, otherwise the
   * context cursor is used and field values can be specified in
   * array 'preset_values. 'msg' takes a message to print if the record
   * was created.
   *
   * @access public
   * @param object application $this
   * @see record_create_set()
   */
  function record_create (&$this)
  {
    $sources =& $this->arg ('sources', ARG_OPTIONAL);
    $msg = $this->arg ('msg', ARG_OPTIONAL);
    $pre = $this->arg ('preset_values', ARG_OPTIONAL);

    # If link was created without arguments, create a source set with a
    # single element from the argument's cursor.
    if (!$sources) {
      $cursor = $this->arg ('_cursor');
      $src = $cursor->source ();
      $sources[$cursor->type ()][$src] = $pre ? $pre : array ();
    } else
      if ($this->arg ('preset_values', ARG_OPTIONAL))
        die ("record_create(): Arguments 'preset_values' and 'sources' " .
             "can't be used together.");

    $key =& record_create_set ($this, $sources);
    return _record_create_continue ($this, $key);
  }

  /**
   * Calls form_update() and outputs a success or error message.
   *
   * Non-existing primary keys will be ignored.
   *
   * @access public
   * @param object application $this
   * @see cmd_update (), form_update()
   */
  function record_update (&$this)
  {
    $ui =& admin_panel::instance ();

    $err = form_update ($this);
    if ($err)
      $ui->panic ($err);

    $ui->msgbox ($this->record_messages['record_saved']);
  }

  /**
   * Event handler: Ask to delete records.
   *
   * This handler asks to kill records and then triggers record_delete_force().
   * Event argument '_cursor' can contain the context cursor, an array of
   * cursors can be passed in 'cursor_list' instead. 'yes_view' can contain
   * an event that is triggered if record were deleted, 'no_view' can contain
   * one for the opposite case (the deletion failed).
   *
   * @access public
   * @param object application $this
   * @see record_delete_force()
   */
  function record_delete (&$this)
  {
    $cursor = $this->arg ('_cursor', ARG_OPTIONAL);
    $cursor_list =& $this->arg ('cursor_list', ARG_OPTIONAL);
    $yes_view = $this->arg ('yes_view', ARG_OPTIONAL);
    $no_view = $this->arg ('no_view', ARG_OPTIONAL);

    $ui =& admin_panel::instance ();
    $m = $this->record_messages;
    $tv = $this->event ();

    # Highlight records, check available cursors.
    if (!$cursor_list && !is_array ($cursor_list)) {
      if (!$cursor)
        die ('record_delete(): No cursor.');
      if (!$cursor->key ())
        die ('record_delete(): No key of any record in source ' .
             $cursor->source () . '.');

      # Highlight record.
      $ui->highlight[$cursor->id ()] = '#FFAAAA';
    } else {
      # Cancel deletion if cursor list is empty.
      if (!sizeof ($cursor_list))
        return;

      # Highlight records.
      foreach ($cursor_list as $cursor) {
        $cursor->set_field ('');
        $ui->highlight[$cursor->id ()] = '#FFAAAA';
      }
    }

    if (!$yes_view) {
      if (!$tv->next)
        die ('record_delete(): No argument \'yes_view\' nor next view.');
      $yes_view = $tv->next;
    }
    if (!$no_view) {
      if (!$tv->next)
        die ('record_delete(): No argument \'no_view\' nor next view.');
      $no_view = $tv->next;
    }

    # Create link to real kill.
    $yes =& new event ('record_delete_force');
    $yes->set_next ($yes_view);
    $yes->args['_cursor'] = $this->arg ('_cursor', ARG_OPTIONAL);
    $yes->args['cursor_list'] =& $cursor_list;

    $ui->confirm ($m['delete_ask'], $m['yes'], $yes, $m['no'], $no_view);
  }

  /**
   * Event handler: Delete records.
   *
   * Event argument '_cursor' can contain the context cursor, an array of
   * cursors can be passed in 'cursor_list' instead.
   *
   * @access public
   * @param object application $this
   */
  function record_delete_force (&$this)
  {
    $cursor = $this->arg ('_cursor', ARG_OPTIONAL);
    $cursor_list =& $this->arg ('cursor_list', ARG_OPTIONAL);

    $ui =& admin_panel::instance ();

    if (!$cursor_list) {
      if (!$cursor)
      die ('record_delete_force(): No context cursor or cursor_list.');

      $err = $cursor->delete ();
      if (!$err)
        return;
    } else {
      foreach ($cursor_list as $cursor)
        if ($err = $cursor->delete ())
          break;
    }

    if (isset ($err))
      $ui->panic ($err);

    $ui->msgbox ($this->record_messages['delete_done']);
  }
?>
