<?php
  /**
   * Database search for forms via cursor_sql
   *
   * @access public
   * @module tk_dbisearch
   * @package User interface toolkits
   * @see cursor_sql
   */

  # $Id: dbisearch.php,v 1.60 2002/06/01 23:34:30 sven Exp $
  #
  # Copyright (c) 2000-2002 dev/consulting GmbH
  #                         Sven Klose <sven@devcon.net>
  #
  # This library is free software; you can redistribute it and/or
  # modify it under the terms of the GNU Lesser General Public
  # License as published by the Free Software Foundation; either
  # version 2.1 of the License, or (at your option) any later version.
  #
  # This library is distributed in the hope that it will be useful,
  # but WITHOUT ANY WARRANTY; without even the implied warranty of
  # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  # Lesser General Public License for more details.
  #
  # You should have received a copy of the GNU Lesser General Public
  # License along with this library; if not, write to the Free Software
  # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  require_once 'cursor/merged.class.php';

  define ('TK_DBISEARCH_NOT_SEARCHED', 0);
  define ('TK_DBISEARCH_NOT_FOUND', 1);
  define ('TK_DBISEARCH_FOUND', 2);

  /**
   * Search description object.
   *
   * @access public
   */
  class tk_dbisearch_query {
    var $not_all = '';
    var $empty_fields = 'skip';
    var $base_exp = 'AND';
    var $where = '';
    var $fields = array ();
    var $size = 0;
    var $limit = 0;
    var $offset = 0;
    var $order = '';
    var $_sql_queries = array ();
  }

  /**
   * Initialise the toolkit.
   *
   * @access public
   * @param object application $this
   */
  function tk_dbisearch_init (&$this)
  {
    $this->add_function ('tk_dbisearch');
    $this->raw_views['tk_dbisearch'] = true;
  }

  /**
   * Full-text search event handler.
   *
   * Event argument 'query' must contain a tk_dbisearch_query object.
   *
   * @access public
   * @param object application $this
   */
  function tk_dbisearch (&$this)
  {
    $o = $this->arg ('query');

    $limit =& $o->limit;
    $offset =& $o->offset;
    if ($offset && !$limit)
      $limit = 10;

    _tk_dbisearch_query_sql_cursors ($this, $o);

    # Save query object for tk_dbisearch_get_query_object().
    $this->tk_dbisearch->query =& $o;
  }

  /**
   * Create queried cursor_sql objects.
   *
   * The cursors are stored in array $this->tk_dbisearch->cursors.
   *
   * @access private
   * @param object application $this
   * @param object tk_dbisearch $qo
   */
  function _tk_dbisearch_query_sql_cursors (&$this, &$qo)
  {
    $limit = (int) $qo->limit;
    $not_all = $qo->not_all;
    $offset = (int) $qo->offset;
    $where = $qo->where;
    $q =& $qo->_sql_queries;

    _tk_dbisearch_create_sql_queries ($this, $qo);

    $qo->size = 0;
    $res = array ();
    foreach ($q as $source => $query) {
      if ($where && $query)
        $query = "($query) AND $where";
      else {
        if (!$query && $not_all)
          continue;
        if ($where)
          $query = $where;
      }

      $l = $limit ? "LIMIT $offset,$limit" : '';
 
      # Query complete number of records.
      $cursor =& new cursor_sql ();
      $cursor->set_source ($source);
      $cursor->query ($query);
      $s = $cursor->size ();
      $qo->size += $s;
      if ($limit < 1)
        continue;

      # Step over cursor if offset is beyond number of results.
      if ($offset >= $s) {
        $offset -= $s;
        if ($offset < 0)
          $offset = 0;
        continue;
      }

      # Perform limited query.
      $cursor->query ($query, $l);

      $s = $cursor->size ();
      if ($s) {
        $offset = 0;
        $limit -= $s;
        $this->tk_dbisearch->cursors[] = $cursor;
      }
    }
  }

  /**
   * Create queries for all form fields if not already done.
   *
   * @access private
   * @param object application $this
   * @param object tk_dbisearch $qo
   */
  function _tk_dbisearch_create_sql_queries (&$this, &$query)
  {
    $fields =& $query->fields;
    if ($query->_sql_queries)
      return;

    foreach (array_keys ($fields) as $field)
      _tk_dbisearch_create_query ($this, $query, $field);
  }

  /**
   * Get element from field desribtion or from member variable in query object.
   *
   * tk_dbisearch_query->fields should contain superclasses of the object.
   *
   * @access private
   * @param object application $this
   * @param object tk_dbisearch $qo
   */
  function &_tk_dbisearch_get_field (&$query, $field, $element)
  {
    $q =& $query->fields[$field][$element];

    if ($q)
      return $q;
    return $query->$element;
  }

  /**
   * Create an SQL query for a single form field.
   *
   * The query is stored $query->_sql_queries.
   *
   * @access private
   * @param object application $this
   * @param object tk_dbisearch $query
   * @param string $field Form field name.
   */
  function _tk_dbisearch_create_query (&$this, &$query, $field)
  {
    $def =& $this->db->def;
    $empty_fields = _tk_dbisearch_get_field ($query, $field, 'empty_fields');
    $empty_fields = strtolower ($empty_fields);
    $base_exp = _tk_dbisearch_get_field ($query, $field, 'base_exp');
    $base_exp = strtoupper ($base_exp);
    $sources = $query->fields[$field]['sources'];
    $order  = $query->order;
    $q =& $query->_sql_queries;

    if (isset ($fields) && !is_array ($fields))
      die ("tk_dbisearch: Field '$field': Argument 'fields' is not an array.");
    if ($empty_fields && $empty_fields != 'skip' && $empty_fields != 'match')
      die ("tk_dbisearch: Unknown value '$empty_fields' for empty_fields.");
    if (!isset ($this->named_elements[$field]))
      die ("tk_dbisearch: Unknown form field '$field'.");

    $val = $this->named_elements[$field];
    $val = trim ($val);
    $val = addslashes ($val);

    foreach ($sources as $source => $fieldnames) {
      if (!isset ($q[$source]) && $empty_fields != 'skip')
        $q[$source] = '';

      # Skip empty field matches if so desired.
      if (!$val && $empty_fields == 'skip') {
        if (!isset ($q[$source]))
          $q[$source] = '';
        continue;
      }

      # Search over all fields if none specified by user.
      if (!sizeof ($fieldnames))
        foreach ($def->types ($source) as $f)
          $fieldnames[] = $f['n'];

      if (!is_array ($fieldnames))
        die ("tk_dbisearch(): Field list ist not an array for source " .
             "'$source', field '$field'.");

      # Assemble query for source.
      $qt = '';
      foreach ($fieldnames as $fieldname) {
        if ($qt)
          $qt .= ' OR ';
        if ($val)
          $qt .= "$fieldname LIKE '%$val%'";
        else
          $qt .= "$fieldname=''";
      }

      # Combine multiple queries in same source.
      if (!isset ($q[$source]))
        $q[$source] = '';
      $q[$source] .= $q[$source] ? " $base_exp " : '';
      $q[$source] .= "($qt)";

      # Append ORDER clause.
      if ($order)
        $q[$source] .= " ORDER BY $order";
    }
    $tmp =& $q;
  }

  /**
   * Get cursor with results.
   *
   * @access public
   * @param object application $this
   * @returns object cursor_merged
   */
  function tk_dbisearch_get_results (&$this)
  {
    $p =& admin_panel::instance ();

    $r = tk_dbisearch_has_result ($this);
    if ($r != TK_DBISEARCH_FOUND)
      return;

    # Record cache should not override our results.
    $p->clear_record_cache ();

    $res =& new cursor_merged ();
    $res->query ($this->tk_dbisearch->cursors);
    return $res;
  }

  /**
   * Check if a search was performed or not and if records were found.
   *
   * @access public
   * @param object application $this
   * @returns int Status flags: TK_DBISEARCH_NOT_SEARCHED,
   *              TK_DBISEARCH_NOT_FOUND or TK_DBISEARCH_FOUND.
   */
  function tk_dbisearch_has_result (&$this)
  {
    if (!isset ($this->tk_dbisearch->cursors))
      return TK_DBISEARCH_NOT_SEARCHED;

    if (!$this->tk_dbisearch->query->size)
      return TK_DBISEARCH_NOT_FOUND;

    return TK_DBISEARCH_FOUND;
  }

  /**
   * Get query object of last result.
   *
   * @access public
   * @param object application $this
   * @returns object tk_dbisearch_query.
   */
  function tk_dbisearch_get_query_object (&$this)
  {
    if (isset ($this->tk_dbisearch->query))
      return $this->tk_dbisearch->query;
  }
?>
