<?php
  # $Id: cursor.class.php,v 1.13 2002/06/01 18:33:50 sven Exp $
  #
  # Cursor base class
  #
  # Copyright (c) 2001-2002 dev/consulting GmbH
  #                         Sven Michael Klose (sven@devcon.net)
  #
  # This library is free software; you can redistribute it and/or
  # modify it under the terms of the GNU Lesser General Public
  # License as published by the Free Software Foundation; either
  # version 2.1 of the License, or (at your option) any later version.
  #
  # This library is distributed in the hope that it will be useful,
  # but WITHOUT ANY WARRANTY; without even the implied warranty of
  # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  # Lesser General Public License for more details.
  #
  # You should have received a copy of the GNU Lesser General Public
  # License along with this library; if not, write to the Free Software
  # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  if (defined ('_CURSOR_CLASS'))
    return;
  define ('_CURSOR_CLASS', true);

  require_once 'object/is_a.php';

  /**
   * Cursor base class
   *
   * @access public
   * @package Cursor interfaces
   */
  class cursor {

    var $_source = ''; # Current view's source name.
    var $_key = 0;     # Current record key's value.
    var $_field = '';  # Currently opened field.
    var $_type;        # Name of SSI.
    var $_lower;       # Lower cursor.
    var $_is_good = false;
    var $_current;     # Last read record.
    var $_is_first = false;   # Last record was first but read with current().
    var $_is_queried = false; # query() was called.

    function cursor ($type)
    {
      $this->_type = $type;
    }

    # Query a result set.
    function query ($selection = '', $order = '')
    {
      $this->_is_first = false;
      $ret = $this->_query ($selection, $order);
      $this->_is_queried = $this->_is_good = $ret;
      return $ret;
    }

    # Return first or last get'ed() result in queried set.
    function &current ()
    {
      if (!$this->_is_good)
        return;

      if ($this->_is_queried)
        return $this->_current;

      # Get first record.
      if (!$this->get ())
        return;
      $this->_is_first = true;

      return $this->_current;
    }

    # Return first or next record.
    function &get ($selection = '', $order = '')
    {
      # Query if selection is defined.
      if ($selection)
        $this->query ($selection, $order);
      else if ($this->_is_first) {
        # Return first record get'ed by current().
        $this->_is_first = false;
        return $this->_current;
      }

      if (!$this->_is_good)
        return; # Nothing queried.

      $ret =& $this->_get ();
      $this->_current = $ret;
      $this->_is_good = $ret ? true : false;
      return $ret;
    }

    function size ()
    {
      die ('cursor::size(): Function not implemented by derived class.');
    }

    # Create unique string for position.
    function id ()
    {
      $id = '';
      foreach (array ('_type', '_source', '_key', '_field') as $v) {
        if (isset ($this->$v))
          $id .= $this->$v;
        $id .= "'";
      }
      if (isset ($this->_lower))
        $id .= $this->_lower->id ();
      return $id;
    }

    # Static function: Return new cursor of type $type.
    function &factory ($type)
    {
      $type = "cursor_$type";
      if (!class_exists ($type))
        die ("cursor::factory(): No cursor of class $type defined.");

      $cursor =& new $type ();
      if (!is_a ($cursor, 'cursor'))
        die ("Cursor $type is not of class 'cursor'.");

      return $cursor;
    }

    function set_source ($source)
    {
      $this->_source = $source;
    }

    function source ()
    {
      return $this->_source;
    }

    function set_key ($key)
    {
      $this->_key = $key;
    }

    function key ()
    {
      return $this->_key;
    }

    function set_field ($field)
    {
      $this->_field = $field;
    }

    function field ()
    {
      return $this->_field;
    }

    function type ()
    {
      return $this->_type;
    }

    # Serialise no more than the reference info.
    function __sleep ()
    {
      return array ('_source', '_key', '_field', '_type', '_lower');
    }
  }
?>
